﻿local addon, C = ...

local gameBuild = select(4, GetBuildInfo())
local playerClass = C.myCLASS

local classEnabled = {
	['PRIEST'] = true,
	['WARLOCK'] = true,
	['MAGE'] = true,
	['DRUID'] = true,
	['SHAMAN'] = true,
}

--if not classEnabled[playerClass] then return end

local db
local _G = _G
local format = string.format
local find = string.find
local match = string.match
local pairs = pairs
local floor = math.floor
local tonumber = tonumber
local tsort = table.sort
local type = type
local strsplit = strsplit
local select = select
local UnitClass = UnitClass
local wipe = table.wipe
local UnitLevel = UnitLevel
local GetItemInfo = GetItemInfo
local InCombatLockdown = InCombatLockdown
local GetTime = GetTime
local gsub = string.gsub
local GetItemStats = GetItemStats
local ceil = ceil
local abs = abs

local ITEM_SPELL_TRIGGER_ONEQUIP = ITEM_SPELL_TRIGGER_ONEQUIP
local ITEM_SPELL_TRIGGER_ONUSE = ITEM_SPELL_TRIGGER_ONUSE
	
	
local enabledProfiles = {}

local ALL_FILTER = 'ALL'

local classFilter = { [ALL_FILTER] = true }

for class in pairs(RAID_CLASS_COLORS) do
	classFilter[class] = true
end

local buildFilter = { [ALL_FILTER] = 'All' }
buildFilter[60000] = '6.0.x'
buildFilter[60100] = '6.1.x'
buildFilter[60200] = '6.2.x'
buildFilter[70000] = '7.0.x'
buildFilter[70100] = '7.1.x'
buildFilter[70105] = '7.1.5'
--[[
	
		["CritRating"] = "CritRating",
		["HasteRating"] = "HasteRating",
		["MasteryRating"] = "MasteryRating",
		["Multistrike"] = "Multistrike",
		["Versatility"] = "Versatility",
		["Stamina"] = "Stamina",
			
	
		crit = 0.7597,
		haste = 0.6883,
		multistrike = 0.5809, 
		versatility =  0.4917, 
		mastery = 0.4109, 

]]

local useBlizzardAPI = false
local GemData = {}
local statCompareCache = {	
	{ 0, "CritRating"},
	{ 0, "HasteRating"},
	{ 0, "MasteryRating"},
	{ 0, "Multistrike"},
	{ 0, "Versatility"},
	{ 0, "Stamina"},
	{ 0, 'Intellect' },
	{ 0, 'Agility' },
	{ 0, 'Strength' },
}

local function statCompareSortFunc(x,y)
	if x[1] > y[1] then
		return true
	else
		return false
	end
end

local function GetMaxPresetStat(name)
	
	statCompareCache[1][1] = db.presets[name].crit or 0
	statCompareCache[2][1] = db.presets[name].haste or 0
	statCompareCache[3][1] = db.presets[name].mastery or 0
	statCompareCache[4][1] = db.presets[name].multistrike or 0
	statCompareCache[5][1] = db.presets[name].versatility or 0
	statCompareCache[6][1] = db.presets[name].stamina or 0
	statCompareCache[7][1] = db.presets[name].intellect or 0
	statCompareCache[8][1] = db.presets[name].agility or 0
	statCompareCache[9][1] = db.presets[name].strenght or 0
	
	statCompareCache[1][2] = "CritRating"
	statCompareCache[2][2] = "HasteRating"
	statCompareCache[3][2] = "MasteryRating"
	statCompareCache[4][2] = "Multistrike"
	statCompareCache[5][2] = "Versatility"
	statCompareCache[6][2] = "Stamina"
	statCompareCache[7][2] = 'Intellect'
	statCompareCache[8][2] = 'Agility'
	statCompareCache[9][2] = 'Strength'
	
--	statCompareCache[6][1] = db.presets[name].crit
	
	tsort(statCompareCache, statCompareSortFunc)
	db.presets[name].maxStat = statCompareCache[1][2]
	
--	print('T', statCompareCache[1][1], statCompareCache[1][2])
end

local function RefreshEnabledProfiles()

	gameBuild = select(4, GetBuildInfo())
	playerClass = select(2, UnitClass('player'))
	
	wipe(enabledProfiles)
	for statname, statdata in pairs(db.presets) do	
		if statdata.__enabled and ( statdata.__class == ALL_FILTER or statdata.__class == playerClass ) and ( statdata.__gameBuild == ALL_FILTER or statdata.__gameBuild == gameBuild ) then
			enabledProfiles[statname] = true		
			GetMaxPresetStat(statname)
		end
	end
end

local EquippedItem = {}
local CompareItem = {}

local statsWeight = {}

--local slot = GetInventorySlotInfo(index)
--GetInventoryItemDurability(slot)

--{'PLAYER_ENTERING_WORLD', "UPDATE_INVENTORY_DURABILITY", "MERCHANT_SHOW"}


--[[
0 = ammo
1 = head
2 = neck
3 = shoulder
4 = shirt
5 = chest
6 = waist
7 = legs
8 = feet
9 = wrist
10 = hands
11 = finger 1
12 = finger 2
13 = trinket 1
14 = trinket 2
15 = back
16 = main hand
17 = off hand
18 = ranged
19 = tabard
20 = first bag (the rightmost one)
21 = second bag
22 = third bag
23 = fourth bag (the leftmost one)
]]
--[[
local hasItem, hasCooldown, repairCost = GameTooltip:SetInventoryItem("player", 10);


for i=1, GameTooltip:NumLines() do        
   --local right = _G[GameTooltip:GetName().."TextRight"..i]:GetText()
   local left = _G[GameTooltip:GetName().."TextLeft"..i]:GetText()
   
   print("T", i, left)
   
end
]]

local ITEM_MOD_INTELLECT_SHORT = ITEM_MOD_INTELLECT_SHORT
local ITEM_MOD_INTELLECT = ITEM_MOD_INTELLECT
local ITEM_MOD_STAMINA_SHORT = ITEM_MOD_STAMINA_SHORT
local ITEM_MOD_STAMINA = ITEM_MOD_STAMINA
local ITEM_MOD_CRIT_RATING_SHORT = ITEM_MOD_CRIT_RATING_SHORT
local ITEM_MOD_CR_MULTISTRIKE_SHORT = ITEM_MOD_CR_MULTISTRIKE_SHORT
local ITEM_MOD_VERSATILITY = ITEM_MOD_VERSATILITY
local ITEM_MOD_HASTE_RATING_SHORT = ITEM_MOD_HASTE_RATING_SHORT
local ITEM_MOD_MASTERY_RATING_SHORT = ITEM_MOD_MASTERY_RATING_SHORT
local ITEM_MOD_SPELL_POWER_SHORT = ITEM_MOD_SPELL_POWER_SHORT
local ITEM_MOD_STRENGTH_SHORT = ITEM_MOD_STRENGTH_SHORT
local ITEM_MOD_AGILITY_SHORT = ITEM_MOD_AGILITY_SHORT
local ITEM_MOD_SPIRIT_SHORT = ITEM_MOD_SPIRIT_SHORT
local ITEM_MOD_EXTRA_ARMOR_SHORT = ITEM_MOD_EXTRA_ARMOR_SHORT

local PATTERN_INTELLECT 		= "(%d+%,?%.?%d*) "..ITEM_MOD_INTELLECT_SHORT..'$'
local PATTERN_STAMINA 			= "(%d+%,?%.?%d*) "..ITEM_MOD_STAMINA_SHORT..'$'
local PATTERN_CRIT				= "(%d+%,?%.?%d*) "..ITEM_MOD_CRIT_RATING_SHORT..'$'
local PATTERN_MULTISTRIKE 		= "(%d+%,?%.?%d*) "..ITEM_MOD_CR_MULTISTRIKE_SHORT..'$'
local PATTERN_VERSATILITY 		= "(%d+%,?%.?%d*) "..ITEM_MOD_VERSATILITY..'$'
local PATTERN_HASTE 			= "(%d+%,?%.?%d*) "..ITEM_MOD_HASTE_RATING_SHORT..'$'
local PATTERN_MASTERY 			= "(%d+%,?%.?%d*) "..ITEM_MOD_MASTERY_RATING_SHORT..'$'
local PATTERN_SPELLPOWER 		= "(%d+%,?%.?%d*) "..ITEM_MOD_SPELL_POWER_SHORT..'$'
local PATTERN_STRENGTH 			= "(%d+%,?%.?%d*) "..ITEM_MOD_STRENGTH_SHORT..'$'
local PATTERN_AGILITY 			= "(%d+%,?%.?%d*) "..ITEM_MOD_AGILITY_SHORT..'$'
local PATTERN_SPIRIT			= "(%d+%,?%.?%d*) "..ITEM_MOD_SPIRIT_SHORT..'$'
local PATTERN_EXTRAARMOR		= "(%d+%,?%.?%d*) "..ITEM_MOD_EXTRA_ARMOR_SHORT..'$'

--[[
-- Feet
local INVTYPE_FEED
local FEETSLOT

-- RINGS
local FINGER0SLOT
local FINGER1SLOT
local INVTYPE_FINGER

-- TRINKET
local TRINKET0SLOT
local TRINKET1SLOT
local INVTYPE_TRINKET
]]

local Gem100 ={
	{ 115803, "CritRating", 35 }, -- Critical Strike Taladite
	{ 115804, "HasteRating", 35 }, -- Haste Taladite
	{ 115805, "MasteryRating", 35 }, -- Mastery Taladite
	{ 115806, "Multistrike", 35 }, -- Multistrike Taladite
	{ 115807, "Versatility", 35 }, -- Versatility Taladite
	{ 115808, "Stamina", 35 }, -- Stamina Taladite
	
	{ 115809, "CritRating", 50 }, -- Greater Critical Strike Taladite
	{ 115811, "HasteRating", 50 }, -- Greater Haste Taladite
	{ 115812, "MasteryRating", 50 }, -- Greater Mastery Taladite
	{ 115813, "Multistrike", 50 }, -- Greater Multistrike Taladite
	{ 115814, "Versatility", 50 }, -- Greater Versatility Taladite
	{ 115815, "Stamina", 50 }, -- Greater Stamina Taladite
	
	{ 127760, "CritRating", 75 }, -- Immaculate Critical Strike Taladite
	{ 127761, "HasteRating", 75 }, -- Immaculate Haste Taladite
	{ 127762, "MasteryRating", 75 }, -- Immaculate Mastery Taladite
	{ 127763, "Multistrike", 75 }, -- Immaculate Multistrike Taladite
	{ 127764, "Versatility", 75 }, -- Immaculate Versatility Taladite
	{ 127765, "Stamina", 75 }, -- Immaculate Stamina Taladite
}

local Gem110 = {

	{ 130215, "CritRating", 100 },
	{ 130216, "HasteRating", 100 },
	{ 130217, "Versatility", 100 },
	{ 130218, "MasteryRating", 100 },
	
	{ 130220, 'HasteRating', 150 }, -- http://www.wowhead.com/item=130220
	{ 130219, "CritRating", 150 },
	{ 130221, "Versatility", 150 },
	{ 130222, 'MasteryRating', 150 },
	
	{ 130247, 'Agility', 200 },
	{ 130248, 'Intellect', 200 },
	{ 130246, 'Strength', 200 },
}

local function GetMaxGem(typeStat)

	local list
	
	
	if UnitLevel('player') == 110 then
		list = Gem110
	else
		list = Gem100
	end
	
	local maxVal = 0
	
	if typeStat then
		for i=1, #list do
			if maxVal < list[i][3] and typeStat == list[i][2] then
				maxVal = list[i][3]
			end
		end
	else
		for i=1, #list do
			if maxVal < list[i][3] then
				maxVal = list[i][3]
			end
		end
	end
	
	return maxVal
end

local function CreateString(data, statweight)
	local totalweght = 0
	
	if not data or not statweight then return 0 end
	
	local tempcrit, tempmultistrike, tempmastery, tempversatility, temphaste = 0, 0, 0, 0, 0
	
	if statweight.intellect and statweight.intellect > 0 and data.intellect then		
		totalweght = totalweght + (data.intellect * statweight.intellect)
	end
	
	if statweight.agility and statweight.agility > 0 and data.agility then		
		totalweght = totalweght + (data.agility * statweight.agility)
	end
	
	if statweight.strength and statweight.strength > 0 and data.strength then		
		totalweght = totalweght + (data.strength * statweight.strength)
	end
	
	if statweight.crit and statweight.crit > 0 and data.crit then		
		totalweght = totalweght + ( (data.crit+ tempcrit ) *  statweight.crit )
	end
	
	if statweight.multistrike and statweight.multistrike > 0 and data.multistrike then		
		totalweght = totalweght + ( (data.multistrike + tempmultistrike ) *  statweight.multistrike )
	end
	
	if statweight.mastery and statweight.mastery > 0 and data.mastery then		
		totalweght = totalweght + ( (data.mastery  + tempmastery ) *  statweight.mastery )
	end
	
	if statweight.versatility and statweight.versatility > 0 and data.versatility then		
		totalweght = totalweght + ( (data.versatility + tempversatility ) * statweight.versatility )
	end
	
	if statweight.haste and statweight.haste > 0 and data.haste then		
		totalweght = totalweght + ( (data.haste + temphaste ) *  statweight.haste )
	end
	
	if statweight.spellpower and statweight.spellpower > 0 and data.spellpower then		
		totalweght = totalweght +  (data.spellpower * statweight.spellpower)
	end
	
	return totalweght
end

local function CoundGem(data, data2, statweight)

	if not data or not data2 or not statweight then return 0, 0 end
	
	local totalweght, tempcrit, tempmultistrike, tempmastery, tempversatility, temphaste = 0, 0, 0, 0, 0, 0
	local totalweght2, tempcrit2, tempmultistrike2, tempmastery2, tempversatility2, temphaste2 = 0, 0, 0, 0, 0, 0
	
	if data.gem then
		local value, typeStat
		
		if type(data.gem) == 'string' then
			value, typeStat = strsplit(':', data.gem)
			value = tonumber(value)	
		else
			typeStat = statweight.gemstat or statweight.maxStat
			value = statweight.gemvalue or GetMaxGem(typeStat)
		end
		
		if typeStat == 'CritRating' then
			tempcrit = value
		elseif typeStat == 'HasteRating' then
			temphaste = value
		elseif typeStat == 'MasteryRating' then
			tempmastery = value
		elseif typeStat == 'Multistrike' then
			tempmultistrike = value
		elseif typeStat == 'Versatility' then
			tempversatility = value
		end
	end

	if statweight.crit and statweight.crit > 0 and data.crit then		
		totalweght = totalweght + ( statweight.crit * tempcrit ) 
	end	
	if statweight.multistrike and statweight.multistrike > 0 and data.multistrike then		
		totalweght = totalweght + ( statweight.multistrike * tempmultistrike ) 
	end
	if statweight.mastery and statweight.mastery > 0 and data.mastery then		
		totalweght = totalweght + ( statweight.mastery * tempmastery )
	end	
	if statweight.versatility and statweight.versatility > 0 and data.versatility then		
		totalweght = totalweght + ( statweight.versatility * tempversatility )
	end	
	if statweight.haste and statweight.haste > 0 and data.haste then		
		totalweght = totalweght + ( statweight.haste * temphaste )
	end
	
	if type(data2) == 'table' then
	
		if data2.gem then
			local value, typeStat
			
			if type(data2.gem) == 'string' then
				value, typeStat = strsplit(':', data2.gem)
				value = tonumber(value)	
			else
				typeStat = statweight.gemstat or statweight.maxStat				
				value = statweight.gemvalue or GetMaxGem(typeStat)
			end
			
			if typeStat == 'CritRating' then
				tempcrit2 = value
			elseif typeStat == 'HasteRating' then
				temphaste2 = value
			elseif typeStat == 'MasteryRating' then
				tempmastery2 = value
			elseif typeStat == 'Multistrike' then
				tempmultistrike2 = value
			elseif typeStat == 'Versatility' then
				tempversatility2 = value
			end
		end
		
		if statweight.crit and statweight.crit > 0 and data2.crit then		
			totalweght2 = totalweght2 + ( statweight.crit * tempcrit2 )
		end	
		if statweight.multistrike and statweight.multistrike > 0 and data2.multistrike then		
			totalweght2 = totalweght2 + ( statweight.multistrike * tempmultistrike2 )
		end
		if statweight.mastery and statweight.mastery > 0 and data2.mastery then		
			totalweght2 = totalweght2 + ( statweight.mastery * tempmastery2 )
		end	
		if statweight.versatility and statweight.versatility > 0 and data2.versatility then		
			totalweght2 = totalweght2 + ( statweight.versatility * tempversatility2 )
		end	
		if statweight.haste and statweight.haste > 0 and data2.haste then
			totalweght2 = totalweght2 + ( statweight.haste * temphaste2 )
		end
		
		if data2.gem and data2.gem == true and totalweght2 == 0 then
			totalweght2 = totalweght
		end
	
	end
	
	return totalweght, totalweght2
end

local selected_preset = nil

local loadGem = 0

local hidegametooltip = CreateFrame("Frame")
hidegametooltip:Hide()
local gametooltip = CreateFrame("GameTooltip", "SPTimers_StatsWeight_GameToolTip", nil, "GameTooltipTemplate");
gametooltip:SetOwner( hidegametooltip,"ANCHOR_NONE");
	
local function GetGemData()
	local failed = false
	wipe(GemData)
	
	local gemList = UnitLevel('player') == 110 and Gem110 or Gem100
	
	for i=1, #gemList do
		local itemID, type, value = gemList[i][1], gemList[i][2], gemList[i][3]
	
		local name, link = GetItemInfo(itemID)

		if not name then
			failed = true
		elseif link then	
			gametooltip:SetHyperlink(link)				
			local left = _G[gametooltip:GetName().."TextLeft4"]:GetText()
			if left then
				GemData[#GemData+1] = { left, value, type }
			else
				failed = true
			end
		end
	end
	
	if failed then
		C_Timer.After(1, GetGemData)
	else
--		print('GetData Loaded in ', format('%.1fs', GetTime()-loadGem))
	end
end

function C:InitStatWeight()

	if IsAddOnLoaded('PriestStatWeights') then
			AleaUI_GUI.ShowPopUp(
			   "SPTimers", 
			   'PriestStatWeights is enabled. Delete addon or disable it.', 
			   { name = "Ok", OnClick = function() DisableAddOn('PriestStatWeights'); ReloadUI(); end}, 
			   { name = "Later", OnClick = function() end}		   
			)		
		return 
	end
	
	if not PSW_SVDB then 
		PSW_SVDB = {} 
	end
	
	db = PSW_SVDB
	
	if not db.presets then
		db.presets = statsWeight
	end
	
	for k,v in pairs(statsWeight) do
		if not db.presets[k] then
			db.presets[k] = v
		end
	end
	
	RefreshEnabledProfiles()
	
	C_Timer.After(0.5, function()
		if InCombatLockdown() then
			hidegametooltip:RegisterEvent('PLAYER_REGEN_ENABLED')
			hidegametooltip:SetScript('OnEvent', function()
				hidegametooltip:UnregisterAllEvents()
				loadGem = GetTime()
				C_Timer.After(0.3, GetGemData)
			end)
		else
			loadGem = GetTime()
			C_Timer.After(0.3, GetGemData)
		end
	end)

	db.totalpresets = db.totalpresets or 0
	
	local presets = {		
		name = "Stats Weight Presets",
		order = 1.1,
		expand = false,
		type = "group",
		args = {}
	}

	presets.args.DisableIt = {
		name = "Disable module",
		type = "toggle",
		order = 0.5,
		width = 'full',
		set = function()
			db.disabled = not db.disabled
			RefreshEnabledProfiles()
		end,
		get = function()
			return db.disabled
		end,	
	}
	
	presets.args.New = {		
		name = "",
		order = 1,
		embend = true,
		type = "group",
		args = {}
	}

	presets.args.New.args.CreateNew = {
		name = "Create New",
		type = "execute",
		order = 1,
		set = function()
			db.totalpresets = db.totalpresets + 1
			
			local named = "Preset #"..db.totalpresets
			
			db.presets[named] = {
				__enabled = true,
				__spec = 3,
				__name = named,
				intellect = 0,
				strength = 0,
				agility = 0, 
				crit = 0,
				haste = 0,
				multistrike = 0, 
				versatility =  0, 
				mastery = 0, 
				spellpower = 0, 			
			}
			
			selected_preset = named
			GetMaxPresetStat(named)
			RefreshEnabledProfiles()
		end,
		get = function()
			
		end,	
	}
	
	presets.args.New.args.Select = {
		name = "Select",
		type = "dropdown",
		width = 'full',
		order = 2,
		values = function()	
			local t = {}
			
			for k, v in pairs(db.presets) do
				t[k] = v.__name or k
			end
			
			return t
		end,
		set = function(info, value)
			
			selected_preset = value
			GetMaxPresetStat(value)
			RefreshEnabledProfiles()
		end,
		get = function()
			return selected_preset
		end,	
	}
	
	presets.args.Edit = {		
		name = "Stats",
		order = 2,
		embend = true,
		type = "group",
		args = {}
	}
	
	presets.args.Gens = {		
		name = "Gems",
		order = 3,
		embend = true,
		type = "group",
		args = {}
	}
	
	presets.args.Edit.args.enable = {
		name = "Enable",
		type = "toggle",
		order = 1,
		width = 'full',
		set = function()
			if selected_preset then
				db.presets[selected_preset].__enabled = not db.presets[selected_preset].__enabled
			end
			RefreshEnabledProfiles()
		end,
		get = function()
			if selected_preset then
				return db.presets[selected_preset].__enabled
			end
			
			return false
		end,	
	}
	
	presets.args.Edit.args.Classes = {
		name = "Classes",
		type = "dropdown",
		order = 1.1,
		values = function()
			local t = {}
			for k,v in pairs(classFilter) do
				t[k] = k
			end
			
			return t
		end,
		set = function(info, value)
			if selected_preset then
				db.presets[selected_preset].__class = value
			end
			RefreshEnabledProfiles()
		end,
		get = function()
			if selected_preset then
				return db.presets[selected_preset].__class
			end
		end,	
	}
	
	presets.args.Edit.args.Build = {
		name = "Build",
		type = "dropdown",
		order = 1.2,
		values = buildFilter,
		set = function(info, value)
			if selected_preset then
				db.presets[selected_preset].__gameBuild = value
			end
			RefreshEnabledProfiles()
		end,
		get = function()
			if selected_preset then
				return db.presets[selected_preset].__gameBuild
			end
		end,	
	}
	
	presets.args.Edit.args.Spec = {
		name = "Spec",
		type = "dropdown",
		order = 1.3,
		values = function()
			local t = {}
			t[ALL_FILTER] = 'All'
			for i=1, GetNumSpecializations() do
				local id, name, description, icon, background, role = GetSpecializationInfo(i)				
				local role = GetSpecializationRoleByID(id)
				
				if role then
					t[id] = name
				end
			end
			
			return t
		end,
		set = function(info, value)
			if selected_preset then
				db.presets[selected_preset].__spec = value
			end
			RefreshEnabledProfiles()
		end,
		get = function()
			if selected_preset then
				return db.presets[selected_preset].__spec
			end
		end,	
	}
	
	presets.args.Edit.args.name = {
		name = "Name",
		type = "editbox",
		width = 'full',
		order = 2,
		set = function(info, value)
			if selected_preset then
				local temp = gsub(value, '||', '|')
				
				db.presets[selected_preset].__name = temp
			end
			RefreshEnabledProfiles()
		end,
		get = function()
			if selected_preset then
			
				local temp = db.presets[selected_preset].__name
				
				if temp then
					temp = gsub(temp, '|', '||')
				end
				
				return temp or selected_preset
			end
			
			return ''
		end,	
	}
	
	presets.args.Edit.args.intellect = {
		name = "Intellect",
		type = "editbox",
		order = 3,
		set = function(info, value)
			if selected_preset then
				local num = tonumber(value)
				if num then
					db.presets[selected_preset].intellect = num
				end
			end
			RefreshEnabledProfiles()
		end,
		get = function()
			if selected_preset then
				return db.presets[selected_preset].intellect
			end
			
			return ''
		end,	
	}
	
	presets.args.Edit.args.agility = {
		name = "Agility",
		type = "editbox",
		order = 3,
		set = function(info, value)
			if selected_preset then
				local num = tonumber(value)
				if num then
					db.presets[selected_preset].agility = num
				end
			end
			RefreshEnabledProfiles()
		end,
		get = function()
			if selected_preset then
				return db.presets[selected_preset].agility
			end
			
			return ''
		end,	
	}
	
	presets.args.Edit.args.strength = {
		name = "Strength",
		type = "editbox",
		order = 3,
		set = function(info, value)
			if selected_preset then
				local num = tonumber(value)
				if num then
					db.presets[selected_preset].strength = num
				end
			end
			RefreshEnabledProfiles()
		end,
		get = function()
			if selected_preset then
				return db.presets[selected_preset].strength
			end
			
			return ''
		end,	
	}
	
	presets.args.Edit.args.crit = {
		name = "Crit",
		type = "editbox",
		order = 4,
		set = function(info, value)
			if selected_preset then
				local num = tonumber(value)
				if num then
					db.presets[selected_preset].crit = num
					GetMaxPresetStat(selected_preset)
				end
			end
			RefreshEnabledProfiles()
		end,
		get = function()
			if selected_preset then
				return db.presets[selected_preset].crit
			end
			
			return ''
		end,	
	}
	
	presets.args.Edit.args.haste = {
		name = "Haste",
		type = "editbox",
		order = 5,
		set = function(info, value)
			if selected_preset then
				local num = tonumber(value)
				if num then
					db.presets[selected_preset].haste = num
					GetMaxPresetStat(selected_preset)
				end
			end
			RefreshEnabledProfiles()
		end,
		get = function()
			if selected_preset then
				return db.presets[selected_preset].haste
			end
			
			return ''
		end,	
	}
	--[==[
	presets.args.Edit.args.multistrike = {
		name = "Multistrike",
		type = "editbox",
		order = 6,
		set = function(info, value)
			if selected_preset then
				local num = tonumber(value)
				if num then
					db.presets[selected_preset].multistrike = num
					GetMaxPresetStat(selected_preset)
				end
			end
		end,
		get = function()
			if selected_preset then
				return db.presets[selected_preset].multistrike
			end
			
			return ''
		end,	
	}
	]==]
	presets.args.Edit.args.versatility = {
		name = "Versatility",
		type = "editbox",
		order = 7,
		set = function(info, value)
			if selected_preset then
				local num = tonumber(value)
				if num then
					db.presets[selected_preset].versatility = num
					GetMaxPresetStat(selected_preset)
				end
			end
			RefreshEnabledProfiles()
		end,
		get = function()
			if selected_preset then
				return db.presets[selected_preset].versatility
			end
			
			return ''
		end,	
	}
	
	presets.args.Edit.args.mastery = {
		name = "Mastery",
		type = "editbox",
		order = 8,
		set = function(info, value)
			if selected_preset then
				local num = tonumber(value)
				if num then
					db.presets[selected_preset].mastery = num
					GetMaxPresetStat(selected_preset)
				end
			end
			RefreshEnabledProfiles()
		end,
		get = function()
			if selected_preset then
				return db.presets[selected_preset].mastery
			end
			
			return ''
		end,	
	}
	
	presets.args.Edit.args.spellpower = {
		name = "SpellPower",
		type = "editbox",
		order = 9,
		set = function(info, value)
			if selected_preset then
				local num = tonumber(value)
				if num then
					db.presets[selected_preset].spellpower = num
				end
			end
			RefreshEnabledProfiles()
		end,
		get = function()
			if selected_preset then
				return db.presets[selected_preset].spellpower
			end
			
			return ''
		end,	
	}
	
	presets.args.Gens.args.value = {	
		name = "Value",
		type = "dropdown",
		order = 1,
		values = function()
			if UnitLevel('player') == 110 then
	
				local gemstat = ''
				
				if selected_preset then
					gemstat = db.presets[selected_preset].gemstat or db.presets[selected_preset].maxStat
				end
				
				if gemstat == 'Intellect' or gemstat == 'Agility' or gemstat == 'Strength' then
					return {
						[200] = '+200',
					}
				else
					return {
						[100] = '+100',
						[150] = '+150',
					}
				end
			end
			
			return {
					[35] = '+35',
					[50] = '+50',
					[75] = '+75',
				}
		end,
		set = function(info, value)
			if selected_preset then
				db.presets[selected_preset].gemvalue = tonumber(value)
			end
			RefreshEnabledProfiles()
		end,
		get = function()
			if selected_preset then			
				local typeStat = db.presets[selected_preset].gemstat or db.presets[selected_preset].maxStat
			
				return db.presets[selected_preset].gemvalue or GetMaxGem(typeStat)
			end
			return UnitLevel('player') == 110 and 150 or 75
		end,
	}
	
	presets.args.Gens.args.stat = {	
		name = "Stat",
		type = "dropdown",
		order = 2,
		values = {
			["CritRating"] = "CritRating",
			["HasteRating"] = "HasteRating",
			["MasteryRating"] = "MasteryRating",
			["Multistrike"] = "Multistrike",
			["Versatility"] = "Versatility",
			["Stamina"] = "Stamina",
			['Intellect'] = 'Intellect',
			['Agility'] = 'Agility',
			['Strength'] = 'Strength',
		},
		set = function(info, value)
			if selected_preset then
				db.presets[selected_preset].gemstat = value
			end
			RefreshEnabledProfiles()
		end,
		get = function()
			if selected_preset then
				return db.presets[selected_preset].gemstat or db.presets[selected_preset].maxStat
			end
			return ''
		end,
	}

	
	presets.args.delete = {
		name = "Delete",
		type = "execute",
		order = -1,
		set = function()
			if selected_preset then
				db.presets[selected_preset] = nil
				selected_preset = nil
			end
			RefreshEnabledProfiles()
		end,
		get = function()
			
		end,	
	}
	
	return presets
end

local function GetExistedLine(self, name)
	for i=1, self:NumLines() do        
		
		local left = _G[self:GetName().."TextLeft"..i]:GetText()
		
	--	print('T', left == name, string.match(left, '^'..name))
		
		if left and match(left, '^'..name) then
			return i
		end
	end
	
	return false
end
--[[
function C:PLAYER_ENTERING_WORLD()

end

function C:UNIT_INVENTORY_CHANGED()

end

function C:UPDATE_INVENTORY_DURABILITY()

end

function C:PLAYER_EQUPMENT_CHANGED()

end

function C:MODIFIER_STATE_CHANGED()end
]]
local function SkipText(text)
	
	if not text then
		return false
	end
	
	if find(text, ITEM_SPELL_TRIGGER_ONEQUIP) then
		return false
	elseif find(text, ITEM_SPELL_TRIGGER_ONUSE) then
		return false
	end

	
	return true
end

local curItemStats = {}
local itemStatsSort = {}
local curstats = {}
itemStatsSort[1] = ITEM_MOD_INTELLECT_SHORT
itemStatsSort[2] = ITEM_MOD_STAMINA_SHORT
itemStatsSort[3] = ITEM_MOD_CRIT_RATING_SHORT
itemStatsSort[4] = ITEM_MOD_CR_MULTISTRIKE_SHORT
itemStatsSort[5] = ITEM_MOD_VERSATILITY
itemStatsSort[6] = ITEM_MOD_HASTE_RATING_SHORT
itemStatsSort[7] = ITEM_MOD_MASTERY_RATING_SHORT
itemStatsSort[8] = ITEM_MOD_SPELL_POWER_SHORT
itemStatsSort[11] = ITEM_MOD_SPIRIT_SHORT

local function GetNumberValue(str, pattern)
	local value = match(str, pattern)
	
	if value then
		value = tonumber((value:gsub(',', '')))
	end
	
	return value
end

local linkTypeItems = {	
	['INVTYPE_HEAD']         = { skipMainStat = false, slot = { 1 }, }, -- голова
	['INVTYPE_NECK']         = { skipMainStat = true, slot = { 2 }, },  -- нека
	['INVTYPE_SHOULDER']     = { skipMainStat = false, slot = { 3 }, },  -- плечи
	['INVTYPE_CLOAK']        = { skipMainStat = false, slot = { 15 }, },  -- плащик
	['INVTYPE_ROBE']         = { skipMainStat = false, slot = { 5 }, },  -- грудак
	['INVTYPE_CHEST']		 = { skipMainStat = false, slot = { 5 }, }, -- грудак без платья
	['INVTYPE_BODY']         = { skipMainStat = false, slot = { 4 }, },  -- рубашка
	['INVTYPE_WRIST']        = { skipMainStat = false, slot = { 9 }, },  -- запястие
	['INVTYPE_HAND']         = { skipMainStat = false, slot = { 10 }, },  -- руки
	['INVTYPE_WAIST']        = { skipMainStat = false, slot = { 6 }, },  -- пояс
	['INVTYPE_LEGS']         = { skipMainStat = false, slot = { 7 }, }, -- ноги
	['INVTYPE_FEET']         = { skipMainStat = false, slot = { 8 }, },  -- тапки
	['INVTYPE_FINGER']       = { skipMainStat = true, slot = { 11, 12 }, },  -- кольцо
	['INVTYPE_TRINKET']      = { skipMainStat = true, slot = { 13, 14 }, },  -- тринка
	['INVTYPE_2HWEAPON']     = { skipMainStat = false, slot = { 16, 17 }, }, -- палка
	['INVTYPE_RANGED']       = { skipMainStat = false, slot = { 16, 17 }, }, -- лук
	['INVTYPE_WEAPON']       = { skipMainStat = false, slot = { 16, 17 }, }, -- одноручка
	['INVTYPE_RANGEDRIGHT']  = { skipMainStat = false, slot = { 16, 17 }, }, -- ванда
	['INVTYPE_SHIELD']       = { skipMainStat = false, slot = { 16, 17 }, }, -- щит
	['INVTYPE_HOLDABLE']	 = { skipMainStat = false, slot = { 16, 17 }, }, -- оффхенд
}
	
local function EnabledShowItemStats(link)
	if link then
		local _, _, _, _, _, class, subclass, _, equipSlot = GetItemInfo(link)
		
		if equipSlot and linkTypeItems[equipSlot] then
		
			return true, linkTypeItems[equipSlot].skipMainStat
		end
	
	end
	
	return false
end

local function FilterByMainStat(int, agi, str)
	return not int and not agi and not str
end

local ItemStatsCache = {}

local function GetItemStats_Custom(self)
	local intellect, stamina, crit, multistrike, versatility, haste, mastery, spellpower, strength, agility, spirit, extraarmor, gem
	
	if useBlizzardAPI then
		wipe(curItemStats)
		wipe(curstats)
		local _name, _link = self:GetItem()
		GetItemStats(_link, curstats)
		
		for stat, value in pairs(curstats) do 
			curItemStats[stat] = tonumber(value)
		end
		
		local gem = false
		if curstats['EMPTY_SOCKET_PRISMATIC'] and curstats['EMPTY_SOCKET_PRISMATIC'] == 1 then
			gem = true
		end
		
		
		intellect	= tonumber(curItemStats[ITEM_MOD_INTELLECT_SHORT] )
		stamina 	= tonumber(curItemStats[ITEM_MOD_STAMINA_SHORT] )
		crit		= tonumber(curItemStats[ITEM_MOD_CRIT_RATING_SHORT] )
		multistrike	= nil
		versatility	= tonumber(curItemStats[ITEM_MOD_VERSATILITY] )
		haste		= tonumber(curItemStats[ITEM_MOD_HASTE_RATING_SHORT] )
		mastery		= tonumber(curItemStats[ITEM_MOD_MASTERY_RATING_SHORT] )
		spellpower	= tonumber(curItemStats[ITEM_MOD_SPELL_POWER_SHORT] )
		strength	= tonumber(curItemStats[ITEM_MOD_STRENGTH_SHORT] )
		agility		= tonumber(curItemStats[ITEM_MOD_AGILITY_SHORT] )
		spirit		= tonumber(curItemStats[ITEM_MOD_SPIRIT_SHORT] )
		extraarmor	= nil
		
		for i=1, self:NumLines() do        
			local left = _G[self:GetName().."TextLeft"..i]:GetText()
			if gem == true then			
				for i=1, #GemData do
					local _text, value, sttype =  GemData[i][1],GemData[i][2],GemData[i][3]				
					if left and find(left, _text) then
						gem = format('%d:%s', value, sttype)
						break
					end
				end
			else
				break
			end
		end	

		return intellect, stamina, crit, multistrike, versatility, haste, mastery, spellpower, strength, agility, spirit, extraarmor, gem
	else
	
		wipe(curstats)
		local _name, _link = self:GetItem()
	
		GetItemStats(_link, curstats)
		local gem = false
		if curstats['EMPTY_SOCKET_PRISMATIC'] and curstats['EMPTY_SOCKET_PRISMATIC'] == 1 then
			gem = true
		end
		
		for i=1, self:NumLines() do        
			local left = _G[self:GetName().."TextLeft"..i]:GetText()
			if gem == true then			
				for i=1, #GemData do
					local _text, value, sttype =  GemData[i][1],GemData[i][2],GemData[i][3]				
					if left and find(left, _text) then
						gem = format('%d:%s', value, sttype)
						break
					end
				end
			end
			
			if left ~= '' and SkipText(left) then

				intellect	= tonumber(intellect or GetNumberValue(left, PATTERN_INTELLECT))
				stamina 	= tonumber(stamina or GetNumberValue(left,  PATTERN_STAMINA) )
				crit		= tonumber(crit or GetNumberValue(left,   PATTERN_CRIT))
				multistrike	= nil --tonumber(multistrike or match(left,  PATTERN_MULTISTRIKE) )
				versatility	= tonumber(versatility or GetNumberValue(left,  PATTERN_VERSATILITY))
				haste		= tonumber(haste or GetNumberValue(left,  PATTERN_HASTE))
				mastery		= tonumber(mastery or GetNumberValue(left,  PATTERN_MASTERY))
				spellpower	= tonumber(spellpower or GetNumberValue(left,  PATTERN_SPELLPOWER))
				strength	= tonumber(strength or GetNumberValue(left,  PATTERN_STRENGTH))
 				agility		= tonumber(agility or GetNumberValue(left,  PATTERN_AGILITY))
				spirit		= tonumber(spirit or GetNumberValue(left,  PATTERN_SPIRIT))
				extraarmor	= tonumber(extraarmor or GetNumberValue(left,  PATTERN_EXTRAARMOR))
			end
		end
		
		return intellect, stamina, crit, multistrike, versatility, haste, mastery, spellpower, strength, agility, spirit, extraarmor, gem
	end
end

local function attachItemTooltip(self)
	if db.disabled then return end
	
	local myItem = self:IsEquippedItem()
	local owner = self:GetName()
	local _name, _link = self:GetItem()
	
	local show, skipMainStat = EnabledShowItemStats(_link)
	if not show then return end
	
	local intellect, stamina, crit, multistrike, versatility, haste, mastery, spellpower, strength, agility, spirit, extraarmor, gem = GetItemStats_Custom(self)

	if not CompareItem[owner] then
		CompareItem[owner] = {}
	end
	
	CompareItem[owner].intellect = intellect
	CompareItem[owner].stamina = stamina
	CompareItem[owner].crit = crit
	CompareItem[owner].multistrike = multistrike
	CompareItem[owner].versatility = versatility
	CompareItem[owner].haste = haste
	CompareItem[owner].mastery = mastery
	CompareItem[owner].spellpower = spellpower
	CompareItem[owner].strength = strength
	CompareItem[owner].agility = agility
	CompareItem[owner].spirit = spirit
	CompareItem[owner].extraarmor = extraarmor
	CompareItem[owner].gem = gem
	
	if not skipMainStat and FilterByMainStat(intellect, agility, strength) then return end
	
	local firstLine = true
	
	for name in pairs(enabledProfiles) do
		local statname = name
		local statdata = db.presets[name]
		local current = CreateString(CompareItem[owner], statdata)
		local gem1 = CoundGem(CompareItem[owner], true, statdata)
		current = current + gem1
	--	self:AddLine(statname..": "..floor(current), 1, 1, 1, 1)
	
		if firstLine then
			firstLine = false			
			self:AddLine(' ')
		end
		
		self:AddDoubleLine(( statdata.__name or statname)..":", floor(current), 1, 1, 1, 1, 1, 1, 1, 1)
	end
	
	if not firstLine then
		self:AddLine(' ')
	end
end

local function MouseoverItemTooltip(self)
	if db.disabled then return end
	
	local myItem = self:IsEquippedItem()
	local owner = self:GetName()
	
	local _name, _link = self:GetItem()
	
	local show, skipMainStat = EnabledShowItemStats(_link)
	if not show then return end
	
	local intellect, stamina, crit, multistrike, versatility, haste, mastery, spellpower, strength, agility, spirit, extraarmor, gem = GetItemStats_Custom(self)

	if not CompareItem[owner] then
		CompareItem[owner] = {}
	end
	
	CompareItem[owner].intellect = intellect
	CompareItem[owner].stamina = stamina
	CompareItem[owner].crit = crit
	CompareItem[owner].multistrike = multistrike
	CompareItem[owner].versatility = versatility
	CompareItem[owner].haste = haste
	CompareItem[owner].mastery = mastery
	CompareItem[owner].spellpower = spellpower
	CompareItem[owner].strength = strength
	CompareItem[owner].agility = agility
	CompareItem[owner].spirit = spirit
	CompareItem[owner].extraarmor = extraarmor
	CompareItem[owner].gem = gem
	
	if not skipMainStat and FilterByMainStat(intellect, agility, strength) then return end
	
	if _G['ItemRefTooltip'] and _G['ItemRefTooltip']:IsShown() and CompareItem['ItemRefTooltip'] then
		local firstLine = true
		
		for name in pairs(enabledProfiles) do		
			local statname = name
			local statdata = db.presets[name]
			
			local current = CreateString(CompareItem['ItemRefTooltip'], statdata)
			local myReal = CreateString(CompareItem[owner], statdata)
			
			local gem1, gem2 = CoundGem(CompareItem['ItemRefTooltip'], CompareItem[owner], statdata)
			current = current + gem1
			myReal = myReal + gem2
			
			local dims = current - myReal
			local dims2 = abs(( current*100/myReal ) - 100)
			local dimsstr
			
			if dims2 ~= dims2 then
				dims2 = -1
			end
		
			if current < myReal then
				dimsstr = format("|cFF00FF00+%d%%", ceil(dims2))
			elseif myReal < current then
				dimsstr = format("|cFFFF0000-%d%%", ceil(dims2))	
			else
				dimsstr = format("|cFFFFFFFF+%d%%", 0)	
			end	
			
			
			local existedLine =  GetExistedLine(self, ( statdata.__name or statname))

			if existedLine then
				_G[owner.."TextLeft"..existedLine]:SetText((statdata.__name or statname)..":")
				_G[owner.."TextLeft"..existedLine]:SetTextColor(1,1,1,1)
				_G[owner.."TextRight"..existedLine]:SetText(format("%d%s", floor(myReal), dimsstr))
				_G[owner.."TextRight"..existedLine]:SetTextColor(1,1,1,1)		
			else
				if firstLine then
					firstLine = false					
					self:AddLine(' ')
				end
			
				self:AddDoubleLine(( statdata.__name or statname)..":", format("%d%s", floor(myReal), dimsstr), 1, 1, 1, 1, 1, 1, 1, 1)	
			end
		end
		
		if not firstLine then
			self:AddLine(' ')
		end
	
	else	
		local firstLine = true
		
		for name in pairs(enabledProfiles) do		
			local statname = name
			local statdata = db.presets[name]
			local current = CreateString(CompareItem[owner], statdata)
			local gem1 = CoundGem(CompareItem[owner], true, statdata)
			current = current + gem1
			
			if firstLine then
				firstLine = false				
				self:AddLine(' ')
			end
			self:AddDoubleLine(( statdata.__name or statname)..":", floor(current), 1, 1, 1, 1, 1, 1, 1, 1)
		end
		
		if not firstLine then
			self:AddLine(' ')
		end
	end
end

local function CompareItemTooltip(self, source)

	local myItem = self:IsEquippedItem()
	local owner = self:GetName()

	local _name, _link = self:GetItem()
	
	local show, skipMainStat = EnabledShowItemStats(_link)
	if not show then return end
	
	local intellect, stamina, crit, multistrike, versatility, haste, mastery, spellpower, strength, agility, spirit, extraarmor, gem = GetItemStats_Custom(self)

	if not CompareItem[owner] then
		CompareItem[owner] = {}
	end
	
	CompareItem[owner].intellect = intellect
	CompareItem[owner].stamina = stamina
	CompareItem[owner].crit = crit
	CompareItem[owner].multistrike = multistrike
	CompareItem[owner].versatility = versatility
	CompareItem[owner].haste = haste
	CompareItem[owner].mastery = mastery
	CompareItem[owner].spellpower = spellpower
	CompareItem[owner].strength = strength
	CompareItem[owner].agility = agility
	CompareItem[owner].spirit = spirit
	CompareItem[owner].extraarmor = extraarmor
	CompareItem[owner].gem = gem
	
	if not skipMainStat and FilterByMainStat(intellect, agility, strength) then return end
	
	local firstLine = true
	
	for name in pairs(enabledProfiles) do		
		local statname = name
		local statdata = db.presets[name]
		
		local current = CreateString(CompareItem[source], statdata)
		local myReal = CreateString(CompareItem[owner], statdata)	
		
		local gem1, gem2 = CoundGem(CompareItem[source], CompareItem[owner], statdata)	
		current = current + gem1
		myReal = myReal + gem2
			
		local dims = current - myReal
		local dims2 = abs(( current*100/myReal ) - 100)
		local dimsstr
		
		if dims2 ~= dims2 then
			dims2 = -1
		end
		
		if current < myReal then
			dimsstr = format("|cFF00FF00+%d%%", ceil(dims2))
		elseif myReal < current then
			dimsstr = format("|cFFFF0000-%d%%", ceil(dims2))	
		else
			dimsstr = format("|cFFFFFFFF+%d%%", ceil(0))	
		end	
		
		
		local existedLine =  GetExistedLine(self, ( statdata.__name or statname))

		if existedLine then
			_G[owner.."TextLeft"..existedLine]:SetText((statdata.__name or statname)..":")
			_G[owner.."TextLeft"..existedLine]:SetTextColor(1,1,1,1)
			_G[owner.."TextRight"..existedLine]:SetText(format("%d%s", floor(myReal), dimsstr))
			_G[owner.."TextRight"..existedLine]:SetTextColor(1,1,1,1)
		else
			if firstLine then
				firstLine = false
				
				self:AddLine(' ')
			end
			
			self:AddDoubleLine(( statdata.__name or statname)..":", format("%d%s", floor(myReal), dimsstr), 1, 1, 1, 1, 1, 1, 1, 1)	
		end
	end
	
	if not firstLine then
		self:AddLine(' ')
	end
end

GameTooltip:HookScript("OnTooltipSetItem", MouseoverItemTooltip)
WorldMapTooltipTooltip:HookScript("OnTooltipSetItem", MouseoverItemTooltip)

local function ShoppingTooltipHandler(self)
	if db.disabled then return end
	
	if _G[self:GetName()]:IsShown() then
		local source = 'GameTooltip'
		
		if self == WorldMapCompareTooltip1 or
			self == WorldMapCompareTooltip2 then
			
			source = 'WorldMapTooltipTooltip'
		end
		
		CompareItemTooltip(self, source)
	end
end

ShoppingTooltip1:HookScript("OnTooltipSetItem", ShoppingTooltipHandler)
ShoppingTooltip2:HookScript("OnTooltipSetItem", ShoppingTooltipHandler)

WorldMapCompareTooltip1:HookScript("OnTooltipSetItem", ShoppingTooltipHandler)
WorldMapCompareTooltip2:HookScript("OnTooltipSetItem", ShoppingTooltipHandler)

--[==[
hooksecurefunc('GameTooltip_ShowCompareItem', function(self, anchorFrame)
	local shoppingTooltip1, shoppingTooltip2 = unpack(self.shoppingTooltips);
	
	print(self:GetName())
	print(shoppingTooltip1:GetName(), shoppingTooltip2:GetName())
	print(self:GetItem())
	print(shoppingTooltip1:GetItem())
	print(shoppingTooltip2:GetItem())

end)
]==]

ItemRefTooltip:HookScript("OnTooltipSetItem", attachItemTooltip)
ItemRefShoppingTooltip1:HookScript("OnTooltipSetItem", attachItemTooltip)
ItemRefShoppingTooltip2:HookScript("OnTooltipSetItem", attachItemTooltip)
